/*
 * halt_on_data_range_with_value.c
 *
 * AET Library public API Definitions 
 *
 * Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *

 *  =============================halt_on_data_with_value=======================
 *  Revision Information
 *   Changed: $LastChangedDate: 2013-01-17 08:19:40 -0600 (Thu, 17 Jan 2013) $
 *   Revision: $LastChangedRevision: 10593 $
 */


/*  ===========================================================================
 *  ==                              OVERVIEW                                 ==
 *  ===========================================================================
 *
 *  This Example demonstrates the programmation of an AET generated hardware
 *  watchpoint when a read or write occurs to a specified range of addresses,
 *  with the qualification of a specific value.  In this example, the hardware
 *  watches for a write to the memory location in the middle of the array
 *  memArray1 with with the specific value of 0xF0F0F0F0.  Any other write to
 *  this range will not trigger a halt.   This halt, generated by the AET
 *  hardware, is impreciseand will show a skid of 7-9 cycles past the location
 *  where the variable was was written.
 */

#include <aet.h>
#include <stdio.h>

#define ARRAY_SIZE 256
#define RANGE_LOWER_BOUND 100
#define RANGE_UPPER_BOUND 200
#define TRIGGER_VALUE 0xF0F0F0F0

/* Global Variables */
Uint32 memVariable, memArray1[ARRAY_SIZE];

/* Function Prototypes */
void test_func();

void main()
{
	AET_jobParams params; /* Parameter Structure */
	AET_jobIndex jobNumber;					/* Job Number returned by AETLIB*/

	/*
	 Initialize Parameter Structure
	 */
	params = AET_JOBPARAMS;

	/* 
	 * Set the Watchpoint to halt when a write to MemVariable1 occurs with the 
	 * value 0xF0F0F0F0.  There can be a few cycles of delay when using a
	 * watchpoint of this kind, but the halt should occur a few cycles after 
	 * the write occurs. AET breakpoints are always imprecise because they 
	 * execute when the instruction is retired from the CPU pipeline.The delay
	 * should  be in the neighborhood of 7-9 cycles.
	 */
	 
	params = AET_JOBPARAMS; /* Initialize Job Parameter Structure */

	params.value = TRIGGER_VALUE;
	params.dataStartAddress = (Uint32) &memArray1[RANGE_LOWER_BOUND];
	params.dataEndAddress = (Uint32) &memArray1[RANGE_UPPER_BOUND];
	params.triggerType = AET_TRIG_HALT_CPU;
	params.readWrite = AET_WATCH_READ;
	params.refSize = AET_REF_SIZE_WORD;
	
	/* Initialize AET */
	AET_init();

	/* Claim the AET resource */
	if (AET_claim())
		return;
	
	/* Setup the AET job with selected parameters */
	if (AET_setupJob(AET_JOB_TRIG_ON_DATA_RANGE_WITH_VALUE, &params))
		return;

	/* Remember the index of this job in the job table */
	jobNumber = params.jobIndex;

	/* Tell the user what the job index is */
	printf("The index is %d\n", jobNumber);

	/* Enable AET */
	if (AET_enable())
		return;

	test_func();

	/* Clear AET Job */
	AET_releaseJob(jobNumber);
		
	return;


}

void test_func()
{
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Write trigger value to memory in the array, but below the specified range.  Should not halt here */
	memArray1[RANGE_LOWER_BOUND - 5] = TRIGGER_VALUE;
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Write trigger value to memory in the array, but above the specified range.  Should not halt here */
	memArray1[RANGE_UPPER_BOUND + 5] = TRIGGER_VALUE;
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Write to memory in the range, but with a value other than the specified one.  Should not halt here */
	memArray1[RANGE_LOWER_BOUND + 5] = TRIGGER_VALUE - 1;
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Write to the specified range with the trigger value. Should not halt here */
	memArray1[RANGE_LOWER_BOUND + 10] = TRIGGER_VALUE;
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Read Below Range */
	memVariable = memArray1[RANGE_LOWER_BOUND - 5];
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Read Above Range */
	memVariable = memArray1[RANGE_UPPER_BOUND + 5];
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Read In Range, Wrong Value */
	memVariable = memArray1[RANGE_LOWER_BOUND - 5];
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	/* Read In Range, Right Value - Should halt */
	memVariable = memArray1[RANGE_LOWER_BOUND + 10];
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
	asm(" nop");
}
